#include <at89LP4052.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <rtx51tny.h>
#include <dsw.h>

#define	 COMMAND	0x00
#define  SENDING	0x01
#define	 RECEIVE	0x02

#define	 RX				0x01
#define	 TX				0x02

extern void Init_Serial(void);											//Externe volanie pre inicializaciu RS232

#define	 Token						'^'												//Malo by to byt 0x5E='^'
#define	 SizeOfPacketData 20												//Velkost packetu je od 1..1500 byte
#define	 MyAddrNet				0x00											//Definujem adresu siete 8051
#define	 MyAddr51					0xAB											//Definujem moju adresu, dostanem ju z .ini debuggera-a
#define	 MyBroadcast			0xFF											//Definujem Broadcast-s address-u

typedef xdata struct  																		//Musi byt deklarovana a pouzivana ako je to uvedene v tomto tvare, inac spravne nefunguje !!!
{
 unsigned char Preamble,Dst,Src;
 unsigned int Size;
 unsigned char Data[SizeOfPacketData];
 unsigned char Crc;
}
Packet51;

unsigned char xdata malloc_mempool[2*(sizeof(Packet51)+5)];

unsigned char task_num;

enum state {OFF=0, ON=1};
enum lists {preamblepkt=0, dstpkt=1, srcpkt=2, sizeofpkthi=3, sizeofpktlo=4, headerpkt=(sizeofpktlo+1), crcpkt=(sizeof(Packet51)-1), endpkt=sizeof(Packet51), sizedatapkt=SizeOfPacketData, /*mempoolpkt=sizeof(MemoryPool)*/};

unsigned char Check_CRC_Recv(unsigned char *Block) //reentrant	//Volam rutinu z viacerych taskov ... ale sposobuje chybu ak je malo pamati napr. u AT89LP4052
{
 unsigned int i;
 unsigned char tmp;
 tmp=0x00;
 for(i=0x00; i<(sizeof(Packet51)-1); i++) tmp+=Block[i];				//Scitam vsetky polozky packetu
 tmp=~tmp; tmp+=1;
 return(tmp);
}

unsigned char Check_CRC_Send(unsigned char *Block) //reentrant	//Volam rutinu z viacerych taskov ... ale sposobuje chybu ak je malo pamati napr. u AT89LP4052
{
 unsigned int i;
 unsigned char tmp;
 tmp=0x00;
 for(i=0x00; i<(sizeof(Packet51)-1); i++) tmp+=Block[i];				//Scitam vsetky polozky packetu
 tmp=~tmp; tmp+=1;
 return(tmp);
}

bit send,recv;

void Int_RS232(void) interrupt 4 //using 3
{
 switch(SCON&(RX+TX)) 
  {
	 case			RX	: RI=OFF; recv=ON; isr_send_signal(RECEIVE); break;
	 case			TX	:	TI=OFF; send=ON; isr_send_signal(SENDING); break;
	 default			: TI=RI=OFF; send=recv=ON; isr_send_signal(RECEIVE); isr_send_signal(SENDING); break;
	}
}

sbit ERROR_MEM = P1^0;

Packet51 *Recv_Message,*Send_Message;

void Receive(void) _task_ RECEIVE
{
 unsigned int iin;
 ERROR_MEM=OFF;
 if((Send_Message=(Packet51*)malloc(sizeof(Packet51)))==0) {ERROR_MEM=ON; os_delete_task(SENDING);}
 while(1)
  {
	 switch(os_wait(K_TMO+K_SIG,0xFF,NULL))
	  {
		 case	TMO_EVENT	: 
		 case	SIG_EVENT	: task_num=os_running_task_id();
											if(recv)
											 {
											  recv=OFF;
												if(SBUF==Token) 
												 {
												  ERROR_MEM=OFF;
													iin=0x0000;
													if((Recv_Message=(Packet51*)malloc(sizeof(Packet51)))==0) {ERROR_MEM=ON; os_delete_task(RECEIVE);}
													memset(Recv_Message,'\0',sizeof(Packet51)); 
												 }
												switch(iin++)
												 {
													case	preamblepkt	:	Recv_Message->Preamble=SBUF; break;
													case	dstpkt			: Recv_Message->Dst=SBUF; break;
													case	srcpkt			: Recv_Message->Src=SBUF; break;
													case	sizeofpkthi	: Recv_Message->Size=256*SBUF; break;
													case	sizeofpktlo	: Recv_Message->Size+=SBUF; break;
													case	crcpkt			: Recv_Message->Crc=SBUF;
																							if(Recv_Message->Crc==Check_CRC_Recv((unsigned char*)Recv_Message))
																							 {
																							  recv=OFF;
																							 }	
																							switch(Recv_Message->Dst)
																							 {
																								case MyAddrNet		: 
																								case MyAddr51			:	//Tu ide programovy kod pre moju adresu ...
																								case MyBroadcast	:	
																																		free(Recv_Message); break;
																								default						:	memset(Recv_Message,'\0',sizeof(Packet51)); free(Recv_Message); break;		//Nie je adresa pre mna ...
																							 }
																							break; 
													default						:	Recv_Message->Data[iin-(headerpkt+1)]=SBUF; break;		//Ukladam data do stuktury ...
												}
											 }
											break;
		}
	}
}

unsigned int iout;

void Sending(void) _task_ SENDING
{
 while(1)
  {
	 switch(os_wait(K_TMO+K_SIG,0xFF,NULL))
	  {
		 case	TMO_EVENT	:
		 case	SIG_EVENT	: task_num=os_running_task_id();
											if(send)
											 {
											  send=OFF;
												switch(++iout)
												 {
													case	preamblepkt	:
													case	dstpkt			: SBUF=Send_Message->Dst; break;
													case	srcpkt			: SBUF=Send_Message->Src; break;
													case	sizeofpkthi	: SBUF=high(Send_Message->Size); break;
													case	sizeofpktlo	: SBUF=low(Send_Message->Size); break;
													case	crcpkt			: SBUF=Send_Message->Crc; break;
													case	endpkt			:	iout=preamblepkt;
																							memset(Send_Message,'\0',sizeof(Packet51));
																							os_delete_task(SENDING); 
																							break;
												 default						:	SBUF=Send_Message->Data[iout-headerpkt]; break;
												 }
												}											
											break;
		}
	}
}

void Command(void) _task_ COMMAND
{ 
 init_mempool(&malloc_mempool,sizeof(malloc_mempool)); 
 Init_Serial();
 os_create_task(RECEIVE);
 while(1)
  {
	 switch(os_wait(K_TMO+K_SIG,50,NULL))
	  {
		 case	TMO_EVENT	:	task_num=os_running_task_id();
											Send_Message->Preamble=Token;																					//Pripravujem strukturu na odoslanie ...
											Send_Message->Dst=0xEF;
											Send_Message->Src=MyAddr51;
										  Send_Message->Size=sizeof(Send_Message->Data);
										  strcpy(Send_Message->Data,"DSWout\r\n");
										  Send_Message->Crc=Check_CRC_Send((unsigned char*)Send_Message);
										  iout=preamblepkt; SBUF=Send_Message->Preamble;												//Spustim komunikaciu ...
											os_create_task(SENDING); break;
		 case	SIG_EVENT	:	break;
		}
	}
}